/* 
 * Copyright 2000 by Jarle (jgaa) Aase mailto:jgaa@jgaa.com
 * All rights reserved 
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Jarle (jgaa) Aase
 * 4. The name of Jarle Aase may not be used to endorse or promote products 
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY JARLE (JGAA) AASE  ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL JARLE AASE BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <stdio.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>
#include <getopt.h>
#include <string.h>

#define VERSION "1.0"

const char *myname;
int debug_flag, verbose_flag, ign_8bit_flag, stop_at_nul_flag;

static struct option options[] = {
  {"help", no_argument, NULL, 'h'},
  {"version", no_argument, NULL, 'v'},
  {"debug", no_argument, &debug_flag, 1},
  {"verbose", no_argument, &verbose_flag, 1},
  {"ignore-8bit", no_argument, &ign_8bit_flag, 1},
  {"stop-at-nul", no_argument, &stop_at_nul_flag, 1},
  {NULL, 0, NULL, 0}
  };

void usage(char *bad_arg, char *explain);
void show_help(void);
int scan_file(const char *file_name);


int main(int argc, char *argv[])
{
  int ch = 0;

  if (((myname = strrchr(argv[0], '/')) == NULL) || !*++myname)
    myname = argv[0];

  while((ch = getopt_long(argc, argv, "vh", options, NULL)) >= 0) {
    switch(ch) {
    case 'h':
      show_help();
    case 'v':
      printf("%s: %s\n", myname, VERSION);
      return 1;
    case '?':
      usage(NULL, NULL);
    }
  }

  if (optind >= argc) {
    usage(NULL, "Missing file argument");
  }

  for(; optind < argc; optind++) {
    scan_file(argv[optind]);
  }

  return 0;
}


