/* 
 * Copyright 2000 by Jarle (jgaa) Aase mailto:jgaa@jgaa.com
 * All rights reserved 
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Jarle (jgaa) Aase
 * 4. The name of Jarle Aase may not be used to endorse or promote products 
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY JARLE (JGAA) AASE  ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL JARLE AASE BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <stdio.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <fcntl.h>
#include <time.h>


extern int debug_flag, verbose_flag, ign_8bit_flag, stop_at_nul_flag;

static char read_buf[1024 * 16];
static char line_buf[1024];

int scan_file(const char *file_name)
{
  int fd = 0, current_line = 1, current_msg = 0, seen_body = 0, current_msg_start = 0,
    seen_nl = 0, has_shown_starterr = 0, prev_line_empty = 0;
  char *p, *pp = line_buf, *pp_limit = line_buf + (sizeof(line_buf) - 2), *ppp;
  int bytes = 0;
  struct stat st;
  time_t now = 0;
  size_t curr_pos = 0, curr_msg_start_ofs = 0;

  printf("File \"%s\": ", file_name);

  if ((fd = open(file_name, O_RDONLY)) < 0) {
    printf("ERROR open failed %s\n", strerror(errno));
    return -1;
  }

  if (fstat(fd, &st) != 0) {
    printf("  ERROR: fstat() failed. %s\n", strerror(errno));
  }
  else {
    time(&now);
    if (st.st_mtime > now)
      printf("  WARNING: Modification time (st_mtime) is in the future!\n");
    if (st.st_ctime > now)
      printf("  WARNING: Creation time (st_ctime) is in the future!\n");
    if (st.st_atime > now)
      printf("  WARNING: Last access time (st_atime) is in the future!\n");
  }

  printf("scanning...\n");

  while((bytes = read(fd, read_buf, sizeof(read_buf))) > 0) {
    if (debug_flag)
      printf("  DEBUG: read() %d bytes\n", bytes);

    for(p = read_buf; bytes > 0; bytes--) {
      *pp = *p++;
      ++curr_pos;
      if (*pp == '\n') {
	/*printf("  DEBUG: have nl: current_line = %d, current msg = %d\n", current_line, current_msg);*/
	pp[1] = 0;
	if ((current_line > 1) && !current_msg && !has_shown_starterr) {
	  printf("  ERROR: The spoolfile does not begin with a valit From header!\n");
	  has_shown_starterr = 1;
	}

	if (strncmp("From ", line_buf, 5) == 0) {
	  if (current_msg && !seen_body)
	    printf("  INFO: Message %d starting at line %d does not have a body\n", 
		   current_msg, current_msg_start);
	 
	  ++current_msg;
	  current_msg_start = current_line;
	  curr_msg_start_ofs = curr_pos -1;
	  seen_body = 0;

	  if (verbose_flag)
	    printf("  VERBOSE: Scanning message %d starting at line %d, offset %u\n", 
		   current_msg, current_msg_start, curr_msg_start_ofs);

	  if ((current_line > 1) && !prev_line_empty) {
	    printf("  WARNING: Message %d starting at line %d is not prefixed with a newline\n", 
		   current_msg, current_msg_start);
	  }
 
	} else /* From */ {
	  if (seen_nl && !seen_body)
	    seen_body = 1;
	}

	prev_line_empty = seen_nl ? 1 : 0;
	++current_line;
	seen_nl = 1;
	pp = line_buf;
	*pp = 0;
      }
      else {
	/* Not newline character */
	seen_nl = 0;

	if ((((unsigned char)*pp) < ' ') 
	    && (*pp != '\t')) {
	  /* Look ahead */
	  if ((*pp == 0) && stop_at_nul_flag) {
	    printf("  ERROR: Found NUL character. Giving up at line %d\n", current_line);
	    goto done;
	  }
	  for(ppp = p; (*ppp == *pp) && (ppp < (pp + bytes)) && ((ppp - p) < (pp_limit - pp)); ppp++)
	    ;
	  if (ppp == p)
	    printf("  WARNING: Control character 0x%x in line %d, message %d\n", 
		   *pp, current_line, current_msg);
	  else {
	    printf("  WARNING: %d control characters with value 0x%x in message %d line %d at offset %u\n", ppp - p, *pp, 
		   current_msg, current_line, curr_pos -1);
	    while((p < ppp) && (bytes > 0)) {
	      *pp++ = *p++;
	      --bytes;
	      ++curr_pos;
	      continue;
	    }
	  }
	}

	if (!ign_8bit_flag && !seen_body && ((unsigned char)*pp) > 127) {
	  printf("  WARNING: 8-bit character 0x%x in header at line %d\n", (unsigned int)(unsigned char)*pp, current_line);
	}
    
	if (++pp >= pp_limit) {
	  printf("  WARNING: line %d is too long.\n", current_line);
	  pp = line_buf;
	}
      }
    }
  }

  if (pp != line_buf)
    printf("  ERROR: Spoolfile does not end with a newline!\n");

 done:
  printf("  done\n");
  close(fd);
  return 0;
}
